IMPLEMENTATION MODULE NameLists;

        (********************************************************)
        (*                                                      *)
        (*                Matching lists of names               *)
        (*                                                      *)
        (*  Programmer:         P. Moylan                       *)
        (*  Started:            11 May 2021                     *)
        (*  Last edited:        11 May 2021                     *)
        (*  Status:             Working                         *)
        (*                                                      *)
        (********************************************************)

IMPORT Strings;

(************************************************************************)
(*                         COMPARING TWO LISTS                          *)
(************************************************************************)

PROCEDURE MatchNames (names: ARRAY OF ARRAY OF CHAR;
                            max: CARDINAL;
                                VAR (*IN*) list: ARRAY OF CHAR): CARDINAL;

    (* This is for algorithm negotiation.                               *)
    (* Array names is an array [0..max] of what we support, and list    *)
    (* is a comma-separated list of what the client supports.           *)
    (* Returns the index of the first element of list that we support,  *)
    (* or max+1 if there is no match.                                   *)

    CONST Nul = CHR(0);

    VAR j0, j, k, alg: CARDINAL;

    BEGIN
        (* j is the current position in the list, and j0 is the     *)
        (* starting position in that list.  The current position in *)
        (* names[alg] is k.                                         *)

        j0 := 0;  alg := 0;  k := 0;

        LOOP    (* until RETURN *)

            (* Try to find a match with name at list[j0].   *)

            j := j0;
            WHILE (names[alg][k] <> Nul) AND (list[j] = names[alg][k]) DO
                INC (j);  INC (k);
            END (*WHILE*);

            IF (names[alg][k] = Nul) AND ((list[j] = ',') OR (list[j] = Nul)) THEN

                (* We have a match. *)

                RETURN alg;
            END (*IF*);

            (* Try next algorithm.  *)

            INC (alg);  k := 0;
            IF alg > max THEN

                (* No match found for current list element, move to next. *)

                WHILE ((list[j] <> ',') AND (list[j] <> Nul)) DO
                    INC (j);
                END (*WHILE*);
                IF list[j] = Nul THEN
                    (* No match for anything in list, return failure. *)
                    RETURN alg;
                ELSE
                    j0 := j + 1;
                    alg := 0;
                END (*IF*);
            END (*IF*);

        END (*LOOP*);

    END MatchNames;

(************************************************************************)
(*                        LISTING AN ARRAY OF NAMES                     *)
(************************************************************************)

PROCEDURE ListAll (Names: ARRAY OF ARRAY OF CHAR;  max: CARDINAL;
                                        VAR (*OUT*) list: ARRAY OF CHAR);

    (* Returns a comma-separated list of the strings in the array       *)
    (* Names[0..max].  We assume there is at least one list entry.      *)

    VAR j: CARDINAL;

    BEGIN
        Strings.Assign (Names[0], list);
        IF max > 0 THEN
            FOR j := 1 TO max DO
                Strings.Append (',', list);
                Strings.Append (Names[j], list);
            END (*FOR*);
        END (*IF*);
    END ListAll;

(************************************************************************)

END NameLists.

