IMPLEMENTATION MODULE Services;

        (********************************************************)
        (*                                                      *)
        (*  The services that sit on top of the transport layer *)
        (*                                                      *)
        (*  Programmer:         P. Moylan                       *)
        (*  Started:            14 June 2021                    *)
        (*  Last edited:        10 January 2022                 *)
        (*  Status:             OK                              *)
        (*                                                      *)
        (********************************************************)


IMPORT Strings;

FROM SYSTEM IMPORT ADR;

FROM Consts IMPORT
    (* const*)  SSH_MSG_SERVICE_REQUEST;

FROM PacketIO IMPORT
    (* type *)  IOPort,
    (* proc *)  SendPacket;

FROM VarStrings IMPORT ByteStr, ByteStringPtr, MakeBS;

FROM Inet2Misc IMPORT
    (* proc *)  Swap4;

FROM LowLevel IMPORT
    (* proc *)  Copy;

(************************************************************************)

TYPE
    NameArray = ARRAY Service OF ARRAY [0..15] OF CHAR;

CONST
    Names = NameArray {"none", "ssh-userauth"};

(************************************************************************)
(*                       IDENTIFIERS FOR SERVICES                       *)
(************************************************************************)

(*
PROCEDURE NameOf (S: Service;  VAR (*OUT*) name: ARRAY OF CHAR);

    (* Returns the name of S. *)

    BEGIN
        Strings.Assign (Names[S], name);
    END NameOf;
*)

(************************************************************************)
(*                         REQUESTING A SERVICE                         *)
(************************************************************************)

PROCEDURE RequestService (port: IOPort;  service: Service): BOOLEAN;

    VAR pkt: ByteStr;
        L, M: CARDINAL;
        servicename: ARRAY [0..15] OF CHAR;

    BEGIN
        Strings.Assign (Names[service], servicename);
        L := LENGTH(servicename);
        MakeBS (pkt, 5 + L);
        pkt.data^[0] := SSH_MSG_SERVICE_REQUEST;
        M := Swap4(L);
        Copy (ADR(M), ADR(pkt.data^[1]), 4);
        Copy (ADR(servicename), ADR(pkt.data^[5]), L);
        RETURN SendPacket (port, pkt);
    END RequestService;

(************************************************************************)

END Services.

